# coding: utf-8

'''
Comparaison de vitesse entre les boucles for, map/filer et les list comprehensions.
On utilisera ici le module timeit qui permet de mesurer le temps d'exécution
de petites portions de code.
'''

from timeit import Timer

# ----------
# Paramètres
# ----------

# La liste sur laquelle on travaille
l=range(10000)
# Si ça va trop vite pour pouvoir mesurer, on va répéter l'opération
# un certain nombre de fois:
num_repet = 1000

# -----------------------------------------
# Construction des différents cas à mesurer
# -----------------------------------------
# On construit une liste de cas; chaque élément de la liste sera un
# dictionnaire associant le nom de la technique utilisée (for/map/...)
# au code à exécuter

cas = []

# x+1 pour tous les x

cas.append({
'for' :'''
r = []
for x in l:
    r.append(x+1)
''',
'map': "r = map(lambda x:x+1, l)",
'list comprehension': "r = [x+1 for x in l]"
})

cas.append({
'for' :'''
r = []
for x in l:
    if x%2:
        r.append(x)
''',
'filter': "r = filter(lambda x: x%2,l)",
'list comprehension': "r = [x for x in l if x%2]"
})

cas.append({
'for' :'''
r = []
for x in l:
    if x%2:
        r.append(x+1)
''',
'map/filter': "r = map(lambda x:x+1, filter(lambda x: x%2,l))",
'list comprehension': "r = [x+1 for x in l if x%2]"
})

# ------------
# Vérification
# ------------
# Pour éviter d'avoir fait des erreurs dans les codes ci-dessus, on va vérifier
# qu'ils donnent tous le meme résultat (non demandé dans la donnée)

for i,c in enumerate(cas):
    results = set()
    for name, code in c.iteritems():
        r = None
        exec code
        results.add(tuple(r))
    if len(results) != 1:
        print "Problème dans le cas %d: les résultats diffèrent!" % i

# ------
# Mesure
# ------
# On en vient au timing proprement dit:

print "List length is %d, repeating each operation %d times" % (len(l), num_repet)
for i,c in enumerate(cas):
    print "Case %d:" % (i+1)
    for name, code in c.iteritems():
        t = Timer(setup='from __main__ import l',stmt=code)
        print "%s: %.2fs" % (name,t.timeit(number=num_repet))
    print "***"