# coding: latin-1

# Ncessaire jusqu' python 2.6
from __future__ import with_statement

def memoize(f):
	""" Un dcorateur qui mmoize la fonction qu'il dcore.
	Attention: ceci n'a de sens que si le rsultat de la fonction dcore
	ne dpend que de ses entres!!
	"""
	# On attache un dictionnaire  la fonction dcore...
	f.memotable = {}
	def inner(*args):
		# Lors de l'appel...
		try:
			# ... si le rsultat pour ces entres est dj mmoris, on le retourne...
			return f.memotable[args]
		except KeyError:
			# ... sinon on le calcule, on le mmorise et on le retourne.
			result = f(*args)
			f.memotable[args] = result
			return result
	# Pour faire un dcorateur "bien lev"
	inner.__doc__ = f.__doc__
	inner.__name__ = f.__name__
	inner.__dict__.update(f.__dict__)
	return inner


if __name__ == "__main__":
	from time import sleep, time
	from contextlib import contextmanager

	@contextmanager
	def timing():
		""" Un gestionnaire de contexte qui mesure le temps
		pass dans son contexte
		"""
		start = time()
		yield
		print "*** Time: %.1f" % (time()-start)

	@memoize
	def test(x,y):
		sleep(2)
		return x+y
		
	with timing():
		print test(1,2)
		
	with timing():
		print test(1,2)
		
	with timing():
		print test(1,3)