# coding: utf-8

'''
Une ré-implémentation simplifiée de contextlib.contextmanager.
L'implémentation originale est un peu plus complexe; elle se trouve facilement dans les
sources du module contextlib.
'''

def contextmanager(fun):
	class inner:
		def __init__(self, *args):
			self.iter = fun(*args)
			
		def __enter__(self):
			self.iter.next()
		
		def __exit__(self, type, value, tb):
			try:
				self.iter.next()
				raise RuntimeError("generator didn't stop") 
			except StopIteration:
				pass
		
	return inner

if __name__ == "__main__":
	# Un p'tit test...
	@contextmanager
	def test(name):
		print "** Début de", name
		yield
		print "** Fin de", name
	
	print "Avant"
	with test("test"):
		print "Dedans..."
	print "Après"